\ 	Forth Tools and Notes
\ 	Assad Ebrahim, ebrahim@mathscitech.org
\ 	2013-07-21--Sun--02:43 -- gathering some useful defs
\ 	2014-02-03--Mon--13:29 -- starting anew, systematically
\	2014-03-06--Thu--23:48 -- covering how one would bootstrap a minimalist forth.
\	2014-03-23--Sun--15:21 -- as of now, this is starting to get a bit bloated (its a repository) -- 10K bytes of dictionary code...

\  Description:
\		Holds useful definitions.  
\		Preference is for small language over speed, i.e. for defining as much as possible in terms of Forth itself as opposed to native assembler.

\  Build / Usage environment:
\		Production:  GForth 0.7.0, installed into c:/totalcmd/ToolboxProgramming/gforth 
\		Mimimal: eForth v1.0 running in DOSBox.  http://www.calcentral.com/~forth/forth/   http://www.offete.com/eforth1.html
\			Assembly listing: http://www.calcentral.com/~forth/forth/eforth/e4.src/EFORTH.SHTML
\		Notepad++: configured so F6 runs _update.bat to place current script automatically into gforth/lib_AKE where it is automatically in the include path for GForth
\		Include syntax:   GFORTH SPECIFIC:  	include lib_AKE/tools_AKE.fs
\						  CLASSIC:  			s" lib_AKE/tools_AKE.fs" included

cr cr
." -------------------------------------------------------" cr
." Including... tools_AKE.fs" cr
." To rollback these definitions, type 'forget-tools'"							cr
   marker forget-tools						\ In other forths, use FORGET and the first definition in the script below
." -------------------------------------------------------" cr
cr


\  FORTH IMPLEMENTATION
\		Hardware vs. Software:
\		Forth is typically implemented as a virtual machine (VM) on top of existing hardware.
\		Hardware is typically a register machine with or without an OS.
\		Whilst a dedicated hardware stack (Forth chip) makes for a much simplified implementation of Forth, but the language does not change.

\ 		Memory:
\		The fundamental of computing is memory.  In hardware this is typically some sort of latch capable of storing (write in) and restoring (read out) values.

\		Stacks:
\		Forth (the language) typically presents three stacks:  a data stack (standard stack), a return stack (rstack) and a floating point stack (fstack)
\				To see the stacks:   .s    f.s
\				To put onto the stacks:  1  3.14e
\				To drop off the stacks: .  f.
\			The data stack is typically implemented into cache memory as it is very frequently  to accessed.  
\			You should assume that data stack operations are very fast -- they are OOP --ordinary op-- in Knuth's terminology.  Floating point stack may not be so fast.
\				Standard stack ops: dup, drop, swap, rot

\		A stack in a Forth chip (EXERCISE) is essentially be an electronic buffer with circuitry to
\			(1) move cells up or down the stack, 
\			(2) provide a dedicated TOP-of-stack (tos) register pointer, 
\				: sp@  ( --- adr ) ; 	\ returns TOS pointer  When nothing is on stack, sp@ @ gives 0  [GForth]
\			(3) provide a THIRD location for swapping, and 
\			(4) support elementary combinators (DUP DROP SWAP).  
\		Assuming the stack has a fixed maximum length is reasonable, as Forth HW will likely have such a maximum even if the SW could in principle treat this as an infinite resource. 
\			An operational maximum is depth 4, or 6, certainly not more than 10.  
\			With a finite stack length the programmer would potentially never have to worry about cleanup.  Pushing more onto it just drops stuff off the back.
\				No need to worry about stack-overflow.   The programmer would only need to ensure that items to be used are not more than 3 or 4 deep.
\			Anything more than that and one should consider using memory or a temporary array or refactoring.

\		THE STACK:  
\		A Forth HW implementation:
\			Forth HW might have linear shifting for the main stack from nos to bos, similar circuitry for rstack that works from rnos to reos
\			TOS would potentially be handled using circular shifting circuitry spanning the three special locations: 
\				tos, nos, & rtos allowing any pairwise movement between them, i.e. between T N RT
\			Additional circuitry required would be:
\				LSH shifts everything left from nos to bos
\				RSH shifts everything right from back to nos, over-writing tos with nos
\				rLSH, rRSH left & right shift rstack in the same way
\				N>T  copies nos & overwrites to tos
\      		T>N  copies tos & overwrites to nos
\				N>RT copies tos to third location  (used for SWAP)
\				RT>T copies third location to nos 
\			Then: the elementary operations are implemented in hardware as follows:
\				DROP is: RSH 
\				DUP is:  LSH T>N
\      		SWAP is: N>RT T>N RT>T
\          	>r is: rLSH T>RT RSH
\				r@ is: LSH RT>T 
\          	r> is: r@ rRSH
\	STACK MANIPULATION (data stack, return stack, float stack)
\		There are three stacks: data stack (dstack), return stack (rstack), and floating point stack (fstack)
\			When should rstack be used?  As a temporary dequeuing facility but must be EMPTIED between successive passes of a loop (since rstack stores stores the loop counter) AND emptied before a word's definition ends (since rstack holds the pointer to the next word in the program)
			
\  	So now we have a minimal instruction set of 6 words:  DUP DROP SWAP >r r> r@
\			These can all be implemented very easily in machine language (about 3 lines each)
\				see Code Cookbook\Embed Extend\Forth+ASM\win32forth\assembly.f
	\   	Aside: r@ has stack ( -- a    r: a --- a ), i.e. copy from top of rstack.  Note: cannot just write : r@ r> dup r> ; since this violates the rules for rstack use...
\		Then we can define basic stack operators (combinators)...
		: over ( a b --- a b a ) >r dup r> swap ; 
		: rot ( a b c --- b c a ) >r swap r> swap ;

\		... and other stack words that are not necessary but are sometimes conveniences... 
		: nip ( a b --- b ) swap drop ;
		: tuck ( a b --- b a b ) swap over ;   \ note: if you're using tuck, you could probably swap things around and use over
		: -rot ( a b c --- c a b ) rot rot ;
		: 2swap ( a b c d -- c d a b ) rot >r rot r> ; 
		: 2drop ( a b --- ) drop drop ;
		: 2dup ( a b -- a b a b ) over over ;
		: 3dup ( a b c -- a b c a b c ) >r 2dup r@ -rot r> ;  \ useful when preserving stack before a 3 argument operation
		: 3drop ( a b c -- ) 2drop drop ; 
		: 4drop ( a b c d -- ) 2drop 2drop ;
		: rev  ( a b c --- c b a ) swap rot ;	\ reverses top 3 elements of stack
		: 4rev ( a b c d --- d c b a ) swap 2swap swap ;  \ reverses top 4 elements of stack
\		: rdrop ( ---     r: a --- )  r> drop ;
: swap-under ( a b c -- b a c ) rot swap ;   \ another permutation, like rev, rot, -rot, e
: over-nos ( a b c -- a b a c ) >R over R> ;  \ like over (over tos), but instead over nos
\      	: under ( a b -- a a b ) over swap ;  \ duplicates nos.  equivalently: like over, but places a copy of nos under tos
		: under ( a b -- a a b ) over swap ;   \ mnemonic: nos taken over, except duplicating under tos
		: pick ( ... a0 N --- .. a0 aN ) 1+ cells sp@ + @ ;   \ copies Nth stack element (! note 0 based since F-83) and places it tos  \ useful for .s
\		: roll ( ... now remove it, shift all down, and put it on tos ... ) ;  

\ STACK OPERATIONS
\	arithmetic: + - * / (integer) /mod (quotient and remainder)		f+ f- f* f/
\   boolean and bitwise: and or xor not invert
\ 	bitwise: rshift lshift
\		not (in GForth) simply tests if 0, so -1 not gives 0 and 0 not gives -1, i.e. it does boolean logic reverse.
\		invert does two's complement. 0 invert = -1  1 invert = -2.  In 8 bits, this would be 0 invert = 0xFF   1 invert = 0xFE.  And 0xFF = -1.
\   comparison: < > <= >= = <>     f< f> f<= f>= f= f<>
\	higher arithmetic: pi  (float stack)   1e fexp  (float stack)		f**  (and therefore roots larger than two)   fsqrt
\		: fsqrt ;
		: finvert ( a -- 1/a ) 1e fswap f/ ;   \ reciprocal
		: froot ( a r -- y ) finvert f** ;     \ rth root of a

\ CONDITIONALS
\	if then 
\	if else then
\	while and for ...


\		Software Implementation: 
\			The stack grows toward LOWER memory, so tos has lowest memory, and bos (stack root) has greatest memory adr.  
\ 		Then the first definition before anything goes on the stack is:
here .  \ the absolute start
: cleared?	depth 0> if abort ." Stack must be cleared before tools_AKE are loaded" then ." ok" cr ;  \  this forces the stack to be cleared
cleared?
sp@ constant bos  \ pointer to BOTTOM of stack... the stack grows upwards from here.  bos @ gives 0, but bos cell- gives the bottom *element* of the stack...
\ 		And we can now define:
: depth ( aN...a1 --- aN...a1 N ) sp@ bos swap - cell / ;  \ size of data stack
\		Every item placed on the stack goes in the cell BEFORE tos, i.e. into 		sp@ cell-
\			: push-onto-stack ( val -- )  sp@ cell- ! ;  \ CONCEPTUALLY --- in practice, this doesn't work -- why?
here  92 -  \ starting memory of words in this toolset, adding 92 bytes of memory used for the above definitions...

\  STACK MANAGEMENT
	\ Now we can combine logical and stack to improve our otherwise slightly verbose if...then construction
		: ?.   ( val -- ) dup if dup . then ;
		: ?dup ( a --- a a   or  a )  dup if dup then ;    \ duplicate if non zero.  
			\ ?dup Critique: conditional stack diagram.  But on the other hand any if else then can also be asymmetric.  It depends how you use it...  Support: Simplifies code by keeping asymmetric cleanup right at the test point rather than delayed in in some potentially distant else branch.  Example:  Not using ?dup:   clear ( --- )	depth dup if 0 do . loop else drop then ; 
	\ And we can now rewrite our first useful words		
		: clear. ( <stack> -- ) 
		\ clear stack to display, emitting TOS first.			Notes: [1]  <stack> can be empty.  [2] GForth has clearstack
		depth dup 0> if 0 do . loop else drop then ;
		: clear._alt ( --- ) depth ?dup if 0 do . loop then ;    \ empties stack tos first (right to left)

\		Memory Model
\			In GForth, there are default maximum sizes for the various elements of a Forth system, but these can be overridden at startup.
\			See: file:///c:/totalcmd/__Repository/working-hg-tech/Toolchain/forth/gforth_html/Invoking-Gforth.html#Invoking-Gforth
\				GForth starts up and reads in gforth.fi (file image)
\				To create an image file: file:///c:/totalcmd/__Repository/working-hg-tech/Toolchain/forth/gforth_html/Image-Files.html#Image-Files
\				Structure: primitives are in the Forth engine.  And then there are definitions written in Forth
\				Notes:  startup switch  -m 1G   -m 1M  -m 1T  etc. sets the maximum allocation of *contiguous* memory available to GForth for the dictionary.
\				Words like create w N cells allot   actually reserve N cells in the dictionary
\				ALLOT guarantees contiguous memory working like a stack growing toward higher addresses, so repeated calls to ALLOT extend the stack.
\				ALLOTted memory is freed by calling it with -N,  i.e. ALLOT -N cells frees N cells
\				ALLOCATE N cells requests them directly from the OS heap but not guaranteed to be contiguous between requests.  Memory obtained by ALLOCATE can be freed with FREE.
\				http://www.complang.tuwien.ac.at/forth/gforth/Docs-html/Heap-Allocation.html#Heap-Allocation

\  NOTATION CONVENTIONS:
\		( --- )  stack diagram
\		( [...].. )  input stream  (not yet parsed)	
\		Stack notation:  tos (top-of-stack)  nos (next-on-stack)  bos (bottom-of-stack), prefixed with r or f if referring to the rstack or fstack
\		adr,len  standard string where len contains the number of characters in the string pointed to by adr
\		z-str    counted string where first character of z-str holds the length of the string






\  FLOW CONTROL WORDS:
\	: begin ... again ;  			 \ loop: indefinite loop
\   : begin ... until  ;			 \ loop: runs till TRUE (while false), always runs once
\   : begin ... while ... repeat ;   \ loop: runs till FALSE (while true), may not run at all if condition is never true
\	: do ( B A ) ... loop  ;  		 \ iterate from A to B-1, incrementing by 1.  SO: for a 1-indexed loop: 11 1 do loop  0-indexed: 10 0 do loop.  !! doesn't like B=A
\		I  J  are loop indices  (they use rstack, so you can't inside a loop)
\		+loop jumps the iteration by an increment...
\		?do   handles the B=A case
\	: leave ;  \ lightest -- exits a DO LOOP loop
		\  Key: leave allows not having to carry a full stack all the way down through a nested loop -- you just leave the inner loop
\	: exit  ;  \ exits the word, leaving the stack as is...
		\  Key:  if ... exit then ;  means dont have to mess about with nested if else statements.  Can just write a sequence of short-circuit ifs that each exit the word if true
\	: quit  ;  \ exits the program, but preserves the stack
\	: abort ;  \ exits program, clears the stack, and throws an error


\  DEFINING WORDS are bootstrapped in:
\		: [ ;  \  switch STATE to interpret (execute) mode
\		: ] ;  \  switch STATE to compile (define) mode
\		: DOES> ;   \	when word is encountered in interpret (execute) mode, puts the address of the dictionary-word onto the stack
\		: ; ( -- )  next  [  ;  immediate   \  immediate is a flag that means "never compile, always execute when encountered".  
											\  ; finishes a : dictionary entry by ending with loopback word <next>, then switching back to interpret (execute mode)
\		: : ( -- ) create ( [word-name] )  ]  does> doLIST ;   \ 	Creates dictionary entry & switches to compile mode... everything until ; is compiled.  
															   \ 	When word-name is called run doLIST on the address
\			(Ref: Julian Noble, http://galileo.phys.virginia.edu/classes/551.jvn.fall01/primer.htm )
\		: literal ;   \ e.g. [ 2 cells ] literal    or  variable tmp  4 tmp !   ...  [ tmp @ ] literal 


\  DICTIONARY & PARSING WORDS  (vocabulary & checks)
\		The most common parsing tokens are tok = bl (space) or newline (cr)
\		: word ( tok -- z-str ) ;   \  parses input stream up to token tok; returns z-str holding contents of parsing.  
									\  Usage: bl word hello count type  or bl word .s find drop execute
									\  Note: word appears to use a buffer that hangs around a little longer than the one used by parse.  Hence use <word> in your application.
\		: find ( z-str --- xt t/f ) count sfind ; \  looks up in dictionary the word-name held in z-str.  CONCEPTUALLY.  In actuality, fixes asymmetric stack diagram in sfind .. parses the dictionary to locate the counted string word.  If found, returns address of program (executable token) and returns true.  Otherwise false
\		: parse ( tok --- adr,len ) ; \ like word, but returns standard string  Usage: bl parse hello type   or bl parse .s sfind drop execute
									  \ Note! Parse uses a buffer that is used by the interpreter, so must assume it is volatile and not available for processing.  Word doesn't.
\		: sfind ( adr,len --- xt t/f if found OR 0 ) ; \ --- so not great as it has an asymmetric stack output... same as find, but taking a standard string as input
		: ' ( [word].. -- xt ) bl word find drop ; \ returns executable token of word -- aborts if not found
\		: see ( [word].. --- )  ;  		\ shows the dictionary definition of a word if it exists
\		: words ( -- ) ;  				\  Lists all words (all vocabularies?)
		: checks ( [text< >].. --- ) 	\ A graceful way of checking whether a word is defined -- without clobbering the stack if it isn't.  
										\ From FORTH-83: http://forth.sourceforge.net/std/fst83/fst83chk.fs
			8 spaces 
				begin ( get next word from input stream & test ok )
					bl word dup dup if c@ ( len ) then 
				while ( ... check dictionary ) 
					dup count type space  ( print word... )
					dup find if 2drop	  ( and do nothing if found... )
					else cr ." missing " count type cr 8 spaces  ( otherwise print that its missing... )
					then
				repeat cr drop ;   \  designed to act on a file with lots of words -- tests whether the Forth has them all in its vocabulary

				
\	CHARACTER WORDS
\		: emit ( a -- ) ;      	\ displays to screen the ascii character associated with code a
		: char ( [c].. -- a )  bl parse drop c@ ; 	\ takes following character and provides its ascii code
\			( within a definition use [char] to compile the next character as an ascii code )
\	Note: Older implementations (e.g. eForth) use $ to indicate hex numbers 0x is GForth.
		32 constant bl	  \ ascii $20 (space)  [eForth -- doesn't have constant word (!)]
		: space bl emit ;  \ displays space   [eForth]
		: spaces ( n -- ) 0 do space loop ;  \ displays n spaces (n>0)  [eForth]

		10 constant newline  \ ascii $0A
		: cr ( -- ) newline emit ; \ carriage return: displays newline  [eForth]

		: tab 4 spaces ; 	   \  [eForth: 7 spaces]
		9 constant tabsys  \ $09 system tab character  [7 spaces]
		: .tabsys ( -- ) tabsys emit ; \  displays sys tab char
		
		44 constant comma 	\ ascii $2c comma
		: .comma ( -- ) comma emit ; \ displays comma: 
		0x8   constant backsp    \ backspace delete
		: .del ( -- ) backsp emit ;	\ displays backspace




\  CONCATENATIVE PROGRAMMING 
\	We know that Forth is post-fix.
\	But on occassion, it is nice to have the ability to do prefix or a mix of pre- and post-fix.  E.g. 2 d 6 for the sum of two 6-sided dice.
\  How to do this?
\ 		: def ( <word1 word2> -- )  bl word bl word ( -- word1 word2) execute execute ;    \ DEFER?
\	It is also useful to be able to compile a number (literal) into a definition
\		: literal ( val -- ) ;   \ compile the number into the definition.  Example:  : d ( s -- r ) dice-roll ;  vs.  : d6 ( -- r ) [ 6 ] literal dice-roll ; 


\  STATE VARIABLES
\	The problem with -1 being true is that it becomes IMPLEMENTATION dependent.
\		E.g. -1 in 32 bits is 0xFFFFFF.  But -1 in 8 bits is 0xFF (255d).  So when printing 255d, you don't see -1, you see some non-printable ascii character
\  1 constant true   \  Note: There should be NO difference between true being 1 or -1, provided the implementation value of true is not used.
\  				\ We know that 0= is the test for false.  And everything else is true.  So -1 and 1 should both be true.
		\ The problem with 1 for true is the INVERT no longer is false... you'd have to define NOT to toggle between and 0 and 1
\  0 constant false
: on  ( adr -- ) true swap ! ;
: off ( adr -- ) false swap ! ;
: c-on  ( adr -- ) true swap c! ;
: c-off ( adr -- ) false swap c! ;

		
\ 	ARITHMETIC & LOGIC ( these exist optimised in GForth )  v is value, v' is transformed value, bool is 1 or 0
		: 1+   ( v --- v' )  1  +   ;				\ increment
		: 1-   ( v --- v' ) 1 - ;					\ decrement
		: 1>   ( v -- bool ) 1 > ;
		: 0>   ( v --- bool ) 0 > ;				\ greater than 0?
		: 0=   ( v --- bool ) 0 = ;				\ equals zero?
		: 0<>  ( v --- bool ) 0= invert ;		\ non-zero?
		: 2*   ( v --- 2v )  1 lshift ;			\ double.  note -- bit shifting is faster than multiplication..., and 2 * equivalent to l lshift
        : 2/	( v --- v' ) 1 rshift ;				\ halve (truncate, i.e. approach 0).   note -- bit shifting is faster than multiplication, and 2 / equivalent to 1 rshift
		: mod  ( n f -- r ) 2dup / * - ;           \ compute modulus of n modulo f, returning residue r, i.e. n mod f congruent to r  ( works for n pos or negative )
\		: /mod ( n f -- r q ) 	;					\ return both modulus and quotient ...
\		: if  ;  									\ can be implemented using branching with addresses (jump table):
\  .... 0> [e][t]  ...(if branch instructions) ! save [e] ptr (this address), and fill in else branch instructions then save this adr, go back and fill in [e][t] with the addresses offsets. and then when the code is executing, it is KNOWN that if means check if zero, and if so, jump to [e] otherwise carry on this point till [t]...
\		: endif then ;								\ to assist others from other languages
\ 		: ?do   --- VERY HELPFUL!  ignores the case where the two indices are EQUAL which otherwise leads to runaway looping...
\		: i ( -- ) ;   \ inner loop counter.  Recall that loop counters are stored on rstack
\		: j ( -- ) ;   \ outer loop counter
\		: invert ( bbbb --- ) ;   \ bitwise NOT provided by implementation underlying hardware
\ why is NOT avoided?  Because it can be used to mean 0= or INVERT.  0= tests whether something is FALSE and turns it to TRUE (-1).  INVERT is the bitwise not operator that flips a 0 to 1 and vice versa.  They are both the same on true and false but INVERT does bitwise NOT (two-s complement) on ALL bits.  0 invert ( -- -1)  1 invert ( -- -2)  so this is 2's complement binary bitwise NOT.
		: not ( true_-1 / false_0 --- false_0 / true_-1 ) 0= ;  \ Boolean NOT.  This was in the F-79 standard.  But the F-83 standard broke this.  The F-94 standard stayed neutral.
			\ as opposed to negate, which just puts a minus sign...
\		: negate ( N -- -N )  ;  \ returns the negative of the number N
\		: and    ( a b --- ) ;   \ bitwise AND provided by implementation hardware
\		: or     ( a b --- ) ;   \   one could be minimalist and derive the other BOOLEAN operators from AND and NOT.  or even from NOT AND (NAND).  
		: xor    ( a b --- ) 2dup and invert -rot or and ;	\  a or b and not (a and b)     and there is also another definition...
			\ test:  1 1 0 0 b4 1 0 1 0 b4 xor dup . .b4



\ NUMBER DISPLAY

\	: . ;  \ prints the number on the stack to the display
\   : .s depth 0 ?do . loop ; \ prints the stack **without** altering it in any way... --- this uses pick

\  NUMBERS & BINARY VIEWING WORDS

\ binary viewing tools
32 constant 32BITS
64 constant 64BITS
$88888888 constant 32BITS-spacer 	\ holds nybble boundaries in a 32-bit word so can space binary output for better readability
: emit-bit ( n bit-mask -- ) and 0= [char] 1 + emit ;  \ emits 1 or 0 depending as integer n is <>0 or 0.  WARNING: depends on false being -1
: ?nyb-space ( bit-mask -- bit-mask ) dup 32BITS-spacer and if space then ; \ Tests how far along mask has proceeded and adds space b/w hi & low nybbles of a byte (for output readability).
: msb ( b -- msb ) 1- 1 swap lshift ;  \ returns msb associated with b-bits.  Ex: 4 msb bin8. [1000]
: show-bin  ( n b --- bbbb bbbb ) \ shows arbitrary length words in binary (up to 32 bits), i.e. show b binary bits of number n
	msb begin ( n mask) 2dup emit-bit 1 rshift dup while ?nyb-space repeat  \ emits binary rep from high to low by down-shifting mask to run through bits, adding space after each nyb
	2drop [char] b emit ;  \ output a terminating terminating 'b'		
\ binary outputs by analogy with word: hex. (in Gforth)
: bin32. ( n --- ) 32 show-bin ;
: bin16. ( n --- ) 16 show-bin ;
: bin8. ( n --- ) 8 show-bin ;
: bin4. ( n --- ) 4 show-bin ;
: bin. ( n --- ) bin32. ;

\ binary creation tools		
: make-bin ( b...b N --- val )  \ make a binary number by entering it onto the stack... (lsb on tos) e.g. 1 1 0 1 b4  >> 13  .b4 >> 1101b
	0 tuck do ( b...b code ) swap i lshift or loop ;  
: b8 ( b...b -- val ) 8 make-bin ;
: b4 ( bbbb -- val ) 4 make-bin ;

\ binary related words
: bsz ( n -- bsz )  
	\ size of binary word.  Logic: finds first set bit working down from 32-bit msb
	\ Ex:  32 bsz .  [6]  32 bin8.    1 bsz . [1]  0 bsz . [0]  0xFFFFFFFF bsz . [32]   0x7FFFFFFF  bsz . [31]
	dup 0= if drop 0 
	   else 32BITS msb 32BITS >R begin ( n mask ) 2dup and 0= while 1 rshift R> 1- >R repeat 2drop R> 
	   then ;
: binsum ( n -- )
	\ sum of digits in binary rep of number n
	\ Ex: 1 binsum .  [1]  2 binsum . [1]  3 binsum . [2]  0xFFFFFFFF binsum . [32]  0x7FFFFFFF binsum . [31]
	32BITS msb 0 >R begin ( n mask ) 2dup and 0<> R> swap - >R 1 rshift dup 0= until 2drop R> ;

\ logarithm logic design choices:
\	** log2f  a) counting how many right-shifts N needs (division by 2) until it hits 0.  N>0
\	b) counting how many left-shifts 1 needs until it exceeds N.
\	** log2c  c) if binary sum > 1 then it cannot be a power of 2.  If bin sum = 1 then it has to be a power of 2.  Use this to do this 
: log2f ( n -- floor[log2[n]] ) 0  begin ( N>>2... counter ) 1+ swap 1 rshift tuck 0= until nip 1- ;
: log2 log2f ;  \ for backwards compatibility
: log2c ( n -- ceil[log2[n]] ) dup log2f swap binsum 1 > - ;
\ Ex: 1 log2f .   [0]   2 log2f .  [1]  1024 log2f . [10]  1023 log2f .  [9]  	1025 log2f . [10]
\ Ex: 1 log2c .   [0]   2 log2c .  [1]  1024 log2c . [10]  1023 log2c .  [10]  	1025 log2f . [11]
\ NOTE: ceiling is not just floor + 1, because that fails when log2f is exact.


\ Exercise: write a display mode that prints the HEX form of a number.  So 15 h  -->  ff 
\ E2: then address the little endian byte order interpreting words in Intel memory...

\ not used
\  $80000000 constant msb32 \ and-mask to pick off most significant bit ... equiv to: 1 31 lshift
\  $ff constant byte-mask  \ and-mask for extracting least significant byte
\  : not8 ( n --- not-n ) invert ;  \ bitwise not for low 8 bits

\ bit field manipulation (in Forth -- slow) -- EXPERIMENTAL
	: lroll8c ( cell -- byt' carry-flg )  \ lroll-byte-and-carry.  Rolls ONLY lower 8 bits left 1 place and set carry-flg if a high bit rolled around
		dup 1 lshift ( b b< ) swap $80 and 
		if 1 or 1 ( set carry flag ) 
		else 0 then >r ( save carry flag )
		$ff and r> ( restore carry ) ; 

	: lroll8 ( byt -- byt' )  \ roll 8 bits left 1 place
		lroll8c drop ;  
	\ note: $80, $8000, $8000 0000 are check masks for 8-, 16-, and 32-bit 'roll' operations.
	\ alternatively, could have used CODE and implement this using Intel chip ROL and ROR (roll right and left) op-codes.  But this suffices.  (Code profiling would show how much the CODE solution was faster by.)

\ Example: An ABACUS

	\ Given a cell PDX (for packed data index), pdx-inc illustrates how to define a custom operation for a custom data structure where low byte is a cycle with 8 values, and high order 3 bytes is the usual binary decimal...  So this is effectively 8 fine tune settings to one click on the main tuning knob.  
	\ Uses: slide rule, knobs, etc.

	variable pdx-mask 1 pdx-mask !    \  special packed (bit-array) index: low byte tracks the bit-index (bix).  high 3 bytes track the byte-index (byx), i.e. holds wptr.
							\ starts at 1 because bix has 8 positions that a 1 can take.

	: pdx-inc ( pdx -- pdx' )  	\ increment operation for the special bit-field data structure index
								\ rolls the low byte until a carry, then increments the high 3 bytes, otherwise updates only the low byte
								\ in this sense it works like an ABACUS
		dup lroll8c if swap 8 rshift 1+ 8 lshift else swap $ffffff00 and then or ;
	: pdx-test ( pdx N -- pdx' ) 1+ 0 \ show pdx and the next N pdxs.  Defaults: pdx=1, N=20
		do I 
			cr ." [" . ." ]   " dup . dup bin16.
			pdx-inc loop ;  

	: byx  		( pdx -- byx )  8 rshift ;  						\ obtain high 3 bytes of pdx  
																	\ in Sieve5, this holds wptr -- which at 24 bits, can index in excess of 16M bytes, referencing 64M w elements
	: bix  ( pdx -- bixN )  $ff and ;								\ obtain lo byte of pdx -- this can hold either bix-mask (1 indexed) or bixN (0-indexed)	
	: unpack-pdx ( pdx -- bix-mask byx ) dup bix swap byx ;
	: pack-pdx  ( bix-mask byx -- pdx ) 8 lshift swap or ;  
	: log2 ( bix-mask -- bix-N ) 0  begin 1+ swap 1 rshift swap over 0= until nip 1- ;	\ get a bit-index bixN from a bit-mask


\	MEMORY

\  WANT:  forget

		4 constant cell   	\ if 32 bit OS	  cell allows independence from word size of the CPU
\		8 constant cell 	\ if 64 bit OS
		: cells ( n -- bytes ) cell * ;		    \ sz in bytes when given length in cell units  (optimised in GForth)
		: cell+ ( adr -- next-cell ) cell + ;   \ advance one cell (optimised in GForth)
		: cell- cell - ; \ reverse one cell
		: char+ ( adr -- next-char ) 1+ ;		\ advance one ascii char (byte if ascii) (optimised in GForth)
	\ Byte words are here to make logic around storage length clear -- self-documenting code
		: bytes ( n -- bytes ) ;				\ sz in bytes of n cells, same as n 1 * 
		: byte+ ( adr -- next-byte ) 1+ ;
		: byte- ( adr -- prev-byte ) 1- ; 

\  Note:  Forth establishes a convention based on ! ( val dest -- )      when dealing with memory storage, the order of arguments is src dest
\		: !  ( val adr -- ) ;
\		: @  ( adr -- val ) ;
\		: ?  ( adr -- ) @ . ; 	\ display value in address, i.e. dereference pointer and display
\		: c! ( val adr -- ) ;
\		: c@ ( adr -- val ) ;

\		: create ( <name>,   --- ) CORE  this parses the input stream for the next word and makes a dictionary entry with it as the label and the subsequent current memory position (here) placed in the definition so that executing the word returns the memory location that was free after the dictionary entry was created and to which any subsequent memory storage would be associatedl
		: , ( val -- ) here cell allot ! ; 	 	\ store val into current location and protect it by advancing the here pointer
		: variable ( <name>, -- )  create 0 , ;  \ initialised variable: cell allot initialised with 0,  create takes "name" from input stream and creates a dictionary definition that refers to the current here memory location.  0 , fills that memory location with 0.  So calling "name" later returns the memory location holding 0.
		: constant ( <name>,  val -- ) create , DOES> @ ;  \ special data structure that hides its address and only returns the value, so cannot be changed.

		: +=  ( val adr -- )  dup @ rot + swap ! ; \ accumulate val into adr, i.e. add val to whatever is aleady held in adr.  (GForth has +!)  += is C style syntax
		: -=  ( val adr -- )  dup @ rot - swap ! ; \ removes val from adr, i.e. subtracts val from whatever is aleady held in adr. -= is C style syntax
			\ increment & decrement operators
		: ++  ( adr -- ) dup @ 1+ swap ! ;  \ increment a variable in memory.  C style syntax
		: --  ( adr -- ) dup @ 1- swap ! ;  \ decrement a variable in memory.  C style syntax
		

\  ARRAYS:

\	Standard forth strings set a convention: pass a pointer and a length, in that order.
\ I have !+ more generally useful (e.g. in the sieve algorithms) because it works naturally with loading from memory
\	 Design Intent:  The following are all intended to work well inside a loop, e.g. processing an array.  No stack accumulation.
		: !+  ( adr val -- adr' )  over ! cell+ ;  			\ store-then-advance one cell.  val stored in adr.  Usage: you provide val inside the loop.  
		: @+  ( adr -- adr' val ) dup cell+ swap @  ; 		\ fetch-then-advance one cell   					 Usage: you handle val inside the loop.
						\ fetches from address, increments address, and leaves received value on TOS ready for further work
		: .+  ( adr -- adr' ) dup @ . cell+ ;  				\ print-then-advance one cell	 Displays within a loop.  
		: C!+ ( adr val -- adr' ) over c! 1+ ;  			\ char.store-then-advance one *char*.   Usage: you provide cval inside the loop.
		: C@+ ( adr -- adr' cval ) dup C@ swap 1+ swap ;  	\ char.fetch-then-advance one *char*.   Usage: you handle cval inside the loop.
		: C.+ ( adr -- adr' ) dup c@ emit 1+ ;  			\ char.print-then-advance one *char*.  Displays within a loop.

		: new-ary ( <name>, N -- )  create cells allot does> ( idx adr -- ) swap cells + ;  \ creates named array of length N, with simple 0-based indexing feature.  Usage: 10 new-ary test  0 test  returns address of test[0], while 10 test returns add of test[10]
		
		\ we could store the size of the string in the first cell, and have the rest of the indices be 1 based...
		\ : fill ( ary Nbytes byte-val -- ) -rot 0 do over c!+ loop 2drop ;  \ fill N bytes of ary with byte-val
		: fillmem  fill ; 	\ avoids ambiguity
		: fillcells ( ary N cell-val -- ) -rot 0 do ( val ary ) over !+ loop 2drop ;   \ fill an array of N *cells* with cell-val
		: filltrue ( ary N --- ) true fillcells ;    \ init memory true (-1)
		: fillfalse ( ary N -- ) 0 fillcells ;		\ init memory with 0 or false
		: printcells ( ary N -- ) 0 cr do .+ loop drop ;  \ print contents of array ary
		: printbytes ( ary N -- ) 0 cr do c@+ . loop drop ; \ print numeric (not char) contents of array of bytes
		
		: cellindex ( ary pos -- k ) swap - cell / ;  \ returns index value associated with pos.  0-indexed.  How to know out of bounds (underflow, overflow, or uninitialised)?
			\ so num elements currently stored in array is:  ary pos index
			\ so max number of elements able to be stored in array is : ary dup ary-offs-max + @ index
		: charindex ( ary pos -- k ) swap - ; 

		\ MLG: http://forth.sourceforge.net/techniques/arrays-mlg/index.html
		\ Michael L Gassanenko
		: [] ( index base -- value ) SWAP CELLS + @ ;
		: []! ( value index base -- ) SWAP CELLS + ! ;
		: []^ ( index base -- addr ) SWAP CELLS + ;
		\ Julian's http://forth.sourceforge.net/techniques/arrays-jvn/index.html

\ ARRAY APPLICATIONS

: t1 ( --- test-stack-1 )	\ a test run
	1 2 3 4 5 6 7 8 9 ;

\ having a scratch area is useful for a variety of algorithms
create tmppad 20 cells allot				\ creates a temporary scratch pad assigned to tmppad holding N cells.  default value for N: 20
: save-stack ( a...z tmppad --- ) depth 1- 0 do tuck ! cell+ loop drop ;   \ save stack to array
: ary-to-stack ( tmppad sz --- z...a ) 0 do dup @ swap cell+ loop drop ;   \ load array to stack
\ for example, reversing the stack using an array
: revN ( a...z tmppad --- z...a )  depth 1- >r  dup >r  save-stack r> r> ary-to-stack ;   \ reverse stack of arbitrary length (up to max tmppad cells) using array memory
	\ Test: tl tmppad revN      each save, load reverses the array...
\ alternatively, using pick and no extra memory, i.e. directly using underlying implementation's memory...
\	: roll  ( aN-1 ... a1 a0 K --- aN-1 ... a1 a0 aK ) \ pick out the kth element of the stack (indexing from 0) and bring to the front
: revNb ( a...z --- z...a ) rev depth 3 do i roll loop ;
\ and if you want to use memory, then you could create it yourself dynamically... but how to free it?  That's why scratch pads are used in embedded programming.  You just use what you've been given.  No more.  So you never have to worry about freeing.  Cause you're keeping track yourself of what you're using...
\ ... but that's all very well for a one-man team and small code... what about when it comes to large code... then how do you keep track?  That's where garbage collection comes in.
\  So --- to save a stack into an array so that reading it out it comes out the right way: you first should store somewhere how long the array is --- and what better place than the first cell of the array --- which leads to 1 indexing...  and then you need to write it in backwards... so that the last element of the stack is into the front of the array.  That way the last element will be read-off first and put at the bottom of the stack...
\ : save-stack ( aN...a1 tmppad --- ) depth dup >r roll \ write in reverse order

	
\	STRINGS in Forth are of three types:  an adr,len pair (standard), a counted string also called Z-string len,adr, or a C-string null terminated...
		: count ( z-str -- adr len ) C@+ ;   \ converts a Z-string to a standard string e.g. $0a word Hello World! count type
		: type ( adr len -- ) 0 do C.+ loop drop ;	\ displays a str,len pair  e.g. $0a parse Hello World! type


\  INPUT WORDS & PAUSE
\	: key ( -- asc ) ; \ returns the ascii code of the key pressed
\	: parse ( c -- adr len ) ; \ returns the standard string holding a string up to the token c
\	: word ( c -- z-str ) ; \ same as parse but returns a counted string
	\ Exploratory key mappings
	: 1key 	( --- ) key clear. ;			\	e.g. a  97d   ctrl-a  1d	esc 27    [ 91   shift-a  65
	: 2keys ( --- ) key key swap clear. ;	\	e.g. 		  alt-a   27 97 	--- NOTE: alt-characters are essentially key pairs: ESC+key... 
	: 3keys ( --- ) key key key rev clear. ;	\	e.g. 		  up-arrow, down-arrow, rt-arr, lt-arr, etc.	27 91 65    or 27 91 66  --- NOTE: arrows are triples: ESC+[+SHIFT-A
	: 4keys ( --- ) key key key key revN clear. ; \	e.g.   f1...f5   e.g. 27 91 91 65    also  pg-up, pg-dn, hm, end, delete, insert  (and weirdly pause and scroll)
	: 5keys ( --- ) key key key key key revN clear. ; \  e.g. f6...f12  e.g.  27 91 49 55 126  
	: pause ( -- ) ." Press any key to continue." key drop ;
	

\  FILE WORDS
\ file handles
0 variable (fh-in) (fh-in) !
0 variable (fh-out) (fh-out) !
: fh-in (fh-in) @ ;
: fh-out (fh-out) @ ;
2 constant r/w
0 constant r/o

\	: throw ( err -- ) if abort then ;  \ nothing if no error, i.e. false, abort if error

\ text proc buffer
256 constant line-max
create line-buf line-max 2 + allot  \ bytes

: open-read    ( s-str -- ) r/o   open-file throw (fh-in) ! ;
: open-write   ( s-str -- ) r/w   create-file throw (fh-out) ! ; \ NOTE: clobbers if exists, creates if does not
: close-input  ( -- )  fh-in  close-file throw ;
: close-output ( -- )  fh-out close-file throw ;

\ : read-line ( buf-dst  buf-sz  fh -- bytes_read  eof_not_yet? excpt ) ; 
: write-line ( buf-src buf-sz  fh -- err )  \ NOTE: it flushes when close-output
	\ GFORTH
	dup >r write-file ?dup
	if r> drop exit
	then
	newline r> write-file ;


: scroll-file ( -- )  
\ display file, one line at a time, with key press to show next line
	begin line-buf line-max fh-in read-line if ." Error!" then ( -- byt ~eof )
		cr line-buf rot type \ display line ( -- ~eof )
	while ( not eof ) key drop repeat ." === End of File!" ;


\  NUMERIC PRINT WORDS

variable holdend
variable holdptr
2142903350 constant holdbuf-end  \ $7FBA1C36
\ THIS IS HORRIBLY COMPLICATED -- IS THERE NOT SOME BETTER WAY TO DO THIS?
: <<# \ GFORTH
	holdend @ holdptr @ - hold holdptr @ holdend ! ;	
: #>> \ GFORTH
	holdend @ dup holdbuf-end u>= -11 and throw count bounds holdptr ! holdend ! ;

: ., ( val -- )  
	\  echo data value comma separated  (see .)
	s>d 	\ convert to double width
	0   	\ no fixed width padded fields
	>r tuck dabs <<# #s rot sign #> r> over - spaces \  WARNING!!  handle the possibility of negative spaces... 
	type 
	.comma space
	#>> ;

: .file ( val -- )
	\  dump data value to file (one number per line) -- no fixed width facility
	s>d tuck dabs <<# #s rot sign #>
	fh-out write-line throw
	#>>  ;

: s.file ( adr,len -- )  \  dump standard string to file
	\  dump string to file
	fh-out write-line throw ;  \ Note: Won't flush to file until close-output

: clear.file
	\  dump stack of values to file (one number per line)
	depth dup if 0 do .file loop else drop then ;


\ LINE INPUT --- you can use expect ( adr n -- )  or query  or word...  and you can use pad as the 64 byte scratchpad... 
create spad 255 cells allot  \ string scratchpad
: s!+ ( ptr char -- ptr' ) over c! 1+ ; 
: getline ( -- sptr len )  \ get line -- note must be less than 256 characters, cr terminates, returns string ready for type 
	\ get chars from keyboard until newline, echoing them as they're typed, and storing them in the scratchpad, returning the length and start of string pointer
	spad >r r@ \ store 
	begin key dup emit tuck s!+ swap $0D = until  
	r@ 1+ - r> swap ;
: hello cr ." What's your name? " getline cr cr ." >> Hello " type cr ;



\ PSEUDO-RANDOM NUMBER GENERATION

\ GGUBS method:  p.56, Scientific Forth, Julian Noble, refers to Bratley, Fox, Schrage, A Guide to Simulation, Springer, 1983, but there are earlier references:
\	  Lewis, Goodman, and Miller, 1969 (LGM algorithm) and Park and Miller (1988) (minimal standard generator)
\    generating formula is recursive
\       r(n+1) := 16807 r(n) MOD m,    m = 2^31 - 1
\	  with r0 > 0 as the seed
\	  This will generate a positive random 32 bit signed integer
\	  This is a class of methods known as   multiplicative congruential methods (without shuffling)
\    Why 16807?  Is it a prime?  No.  16807 = 7^5...
\	  How about 2^31 - 1, is it a prime?  Need a larger prime lookup, for 2,147,483,647
\			Yes.  It is one of only four known double Mersenne primes.  (1772, Euler)  http://en.wikipedia.org/wiki/2147483647
\          Interestingly, it is also the largest signed 32-bit value possible...


16807 constant c  			\ coefficient
-1 1 rshift constant m		\ modulus  (2,147,483,647, i.e. 2^31-1, i.e. greatest 32-bit sign value)
variable seed 				\ holds initial seed for PRNG (pseudo-random number generator).  seed > 0   Subsequently holds last random number generator used.
1 seed !
: lgm ( -- r )  	\ generate next pseudo-random number using GGUBS method
	seed @   		\ obtain r(n)
					\	c * m mod  		\ generate r(n+1) --- this won't work because of INTEGER OVERFLOW!
	c m */mod drop	\  generate r(n+1) using double precision intermediate for the multiplication
	dup seed ! 		\ save r(n+1)
;

: lgm-test ( N -- r )  	\ generate N random numbers using LGM method   PASS if:  r(1000)==522329230  and  r(10,000)=1043618065
	1 seed ! 		\ set seed
	0 do ." . " loop 	\ display dice simulations as working through N random numbers
	cr seed @ . 	\ show r(N)
; 
\ for testing / viewing, could print output and plot...

\  HOW to get uniform floats, i.e. between [0,1]?

: rand-lo ( n -- r ) lgm swap mod 1+ ;  \ generate random integer between 1 and n using MOD on low-order bits  ---- NEED TO PROVE THAT this is adequately random see [NR] for critique
: rand-hi ( n -- r ) lgm 16 rshift swap mod 1+ ;   \ generate random integer from high-order bits...  [NR] recommends this, but won't it give zero more often than anything else?


\ PROFILING TOOLS & TIME&DATE
\	: TIME&DATE ( -- ss mm hh dd MM YYYY ) ;  \ built-in to GForth
\ The cool thing about Forth is that *nothing* is hidden from you.
\ Right down to the hardware.  
\ Want to know how something is done.  It's right there.  Forth is by design open, transparent, non-proprietary.
\ e.g. see time&date
\ this is quite slow and imprecise...
: secs ( -- day-sec ) time&date 3drop 60 * + ( s m ) 60 * + ;   \ seconds elapsed in the day.  NOTE: if you are approaching a day boundary, then the difference won't be useful.
\ secs benchmark secs swap - . .( secs ) 
: benchmark ( ['xt]..  --  sec )  	\ crude elapsed time stopwatch -- not CPU time note.  Note: Assumes xt doesn't leave anything on the stack...
	secs ' execute secs swap - . ." secs" ;  \ Note: if this is not multiple seconds, then you need to loop say 1000 times to get a sensible result
\ this is accurate to about 1 ms
\ test with   : s1000  1000 ms ;  \ sleep for 1 s

\  for more sub-ms timing with rdtsc --- courtesy of Albert van der Horst
\ see c:\totalcmd\__Repository\working-hg-tech\CodeCookbook\_Embed_Extend\Forth+Assembly\assembly.f  
\   right now I have ticks for Win32F --- will see if I can get it for GForth using abi-code...

\ otherwise can use GForths CPUTIME word --- less accurate, more overhead



\ LATER:
\  Can Forth code be packaged into an executable?  Yes, see COOS-HAAK forth...
\  Can it be closed off and made proprietary?  With GForth?


\ >>>>>>>>>>>>  UNTESTED !!!!! <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
\ ARRAYS:   0-indexed, contiguous memory, cell aligned, maintaining its own state (max num elements, count of elements, head, and tail)
\	The thing about arrays is that usually all you need is address and number of elements (i.e. currently present or maximum)
\	And in general, you know what these are when you're in the problem.
\	So the real question is: how much really needs to be extracted.
\ Note: What do we really need with arrays?
\  Iteration
\  Indexing
\  To ensure that we don't overrun...
\ array methods: count, sum, iterate over (without exceeding the boundary), reduce, map?, access by index (but not allowing exceeding the boundary), slice
\ list methods:  append, chop (remove from end), insert (at arbitrary position), delete (at arbitrary position)
\ ary-nxt  should only advance 
\ ?ary-full  whether ary is full or not
\ append-ary  whether arry's can be concatenated or appended to...

cr cr
	." [end tools_AKE.fs]"
cr	." SIZE "
	here swap - dup 8 / . ." CELLS"  ."   (" . ." BYTES)"
cr  ." -------------------------------------------" cr
cr
